width          = 14.5;
wall_thickness = 1.5;
total_width    = width + 2 * wall_thickness;

cavity_diameter       = 19.0;
cavity_radius         = cavity_diameter / 2;
cavity_outer_diameter = cavity_diameter + 2 * wall_thickness;
cavity_outer_radius   = cavity_outer_diameter / 2;

nozzle_length       = cavity_outer_diameter * 1.2;
nozzle_height       = 1.5;
nozzle_outer_height = nozzle_height + 2 * wall_thickness;

opening_protrusion = nozzle_outer_height / 2;
opening_length     = 4.0;


whistle();


module whistle() {
    union() {
        whistle_base();

        translate([0.0, 0.0, wall_thickness])
        whistle_body();

        translate([0.0, 0.0, wall_thickness + width])
        whistle_base();

        holder();
    }
}

module whistle_base() {
    linear_extrude(wall_thickness) {
        cavity_pos();
        nozzle_pos();
    }
}

module whistle_body() {
    linear_extrude(width)
    difference() {
        union() {
            cavity();
            nozzle();
            opening_pos();
        }
        opening_neg();
    }
}

module holder() {
    inner_diameter = 7.0;
    outer_diameter = (inner_diameter / 2 + wall_thickness * 1.5) * 2;

    linear_extrude(total_width)
    difference() {
        angle = 28.0;

        offset_x = cavity_outer_radius * cos(angle);
        offset_y = cavity_outer_radius * sin(angle);

        translate([offset_x, offset_y, 0.0])
        difference() {
            circle2(d = outer_diameter);
            circle2(d = inner_diameter);
        }

        cavity_pos();
        nozzle_pos();
    }
}


module cavity() {
    difference() {
        cavity_pos();
        cavity_neg();
        nozzle_neg();
    }
}

module nozzle() {
    difference() {
        nozzle_pos();
        nozzle_neg();
        cavity_pos();
    }
}


module opening_pos() {
    intersection() {
        translate([-cavity_radius, cavity_radius - nozzle_height / 2, 0.0])
        square([cavity_radius + opening_protrusion, nozzle_height / 2]);
        cavity_pos();
    }
}

module opening_neg() {
    right  = opening_protrusion + opening_length;
    middle = -opening_protrusion;
    bottom = -nozzle_outer_height;

    translate([0.0, cavity_outer_radius, 0.0])
    polygon([
        [0.0, 0.0],
        [opening_protrusion, middle],
        [opening_protrusion, bottom],
        [right, bottom],
        [right, 0.0]
    ]);
}

module cavity_pos() {
    circle2(d = cavity_outer_diameter);
}

module cavity_neg() {
    circle2(d = cavity_diameter);
}


module nozzle_pos() {
    translate([0.0, cavity_outer_radius - nozzle_outer_height, 0.0])
    square([nozzle_length, nozzle_outer_height]);
}

module nozzle_neg() {
    translate([0.0, cavity_radius - wall_thickness, 0.0])
    square([nozzle_length, nozzle_height]);
}


module circle2(d) {
    circle(d = d, $fn = 360);
}
